
// ADC-Demo 12Bit
// Atxmega128A1 und USARTE0 (PIN 37 RXT) und (PIN 38)TXT)  115200 bps
// ADCCH0 PA0 (PIN 95) poitiv, GND negativ  
// Intern 1 Volt 
// unsigned Betrieb 0 - 4095 Samples
// Offset ja


#include <stdio.h>
#include <avr\io.h>
#include <stddef.h>
#include <avr/pgmspace.h>
#include <string.h>
#include "avr_compiler.h"
#include "uart.h"

#include "xitoa.h"

#define NBR_OF_SAMPLES 10 //liest 10 mal den ADC und bildet den Mittelwert


void system_clocks_init(void); 	// 32Mhz/2 =16Mhz
unsigned int get_adValue(void); // ADC init
void run_adc(void);				// ADC run

int main(void)
{
	system_clocks_init();	
	
	//USART USARTE0 Port konf.
  	PORTE.DIRSET = PIN3_bm;  // Pin33 von PORTE3 (TXD0) ist Ausgang                          
  	PORTE.DIRCLR = PIN2_bm;  // Pin32 von PORTE2 (RXD0) ist Eingang                          
 
 
    
    // Init USART 115200bps  ->     
    USARTE0.BAUDCTRLA = 0xD7;   // (F_CPU 16000000 0xD7/0x93),(F_CPU 32000000  0x17/A4)
    USARTE0.BAUDCTRLB = 0x93;   
    USARTE0.CTRLA = 0x10;       // RX is low level interrupt	 
    USARTE0.CTRLC = 0x03;       // Async, No Parity, 1 stop bit, 8 data bits
    USARTE0.CTRLB = 0x18;       // Enable RX and TX // kein CLK2X  !!!!!
             // Initialize Send and Receive buffers

	// alle Interupt einschalten
	PMIC.CTRL |= PMIC_HILVLEN_bm; 
	PMIC.CTRL |= PMIC_MEDLVLEN_bm;
	PMIC.CTRL |= PMIC_LOLVLEN_bm;
	
	PORTB.DIRTGL   |= PIN0_bm;  	// Toggle LED nach ADC-Wandlung

	sei(); 
	
	uart_init();       
	xfunc_out = (void (*)(char))uart_put; // xfunc Ausgabe  mit usart verbinden	

	
	xprintf(PSTR("ADC_Test.. \r")); //Testausgabe

  	ADCA.CTRLA |= 0x1;                                   // enable adc
  	ADCA.CTRLB = ADC_RESOLUTION_12BIT_gc;                // 12 bit conversion
  	ADCA.REFCTRL = ADC_REFSEL_INT1V_gc | 0x02;           // internal 1V bandgap reference
  	ADCA.PRESCALER = ADC_PRESCALER_DIV16_gc;             // peripheral clk/16 (16MHz/16=1Mhz)
  	ADCA.CH0.CTRL = ADC_CH_INPUTMODE_SINGLEENDED_gc;     // single ended
  	ADCA.CH0.MUXCTRL = ADC_CH_MUXPOS_PIN0_gc;            // PORTA:0 gegen GND
	
	while(true)
	  {				// main loop
	 delay_s(1); 	// delay 1 Sekunde 
	 run_adc();
	 //nop();

	  }
}


void run_adc(void)
{
volatile uint16_t ad_wert; 
   		 

   	ad_wert = get_adValue(); 
 
   	rprintfFloat(3,ad_wert *(1.0/4095.0));		// 3.stellig Float
	 
	xprintf(PSTR("\r")); // neue Zeile

}



/* Lese vom  A/D Converter  return sum */ 
unsigned int get_adValue(void) 
{ 
   uint16_t tmp, sum = 0; 
   uint8_t i;

   for (i=0;i<NBR_OF_SAMPLES;i++)
   	{ 

  	// trigger single conversion
  	ADCA.CTRLA |= 0x4; 
  	// wait for result
  	while(!ADCA.CH0.INTFLAGS);                 // wait for conversion complete flag
  	ADCA.CH0.INTFLAGS=ADC_CH_CHIF_bm;          // clear int flags (cleared by writing 1)

    tmp = ADCA.CH0RESL; 
    tmp += ADCA.CH0RESH<<8; 
    sum += tmp;      	
	} 

   sum /= NBR_OF_SAMPLES; 
    
   
   PORTB.OUTTGL |= PIN0_bm;  	// Toggle LED   
  
   return  sum; 
}

// System Clocks initialization
void system_clocks_init(void)
{
unsigned char n,s;

// Save interrupts enabled/disabled state
s=SREG;
// Disable interrupts
cli();

// Internal 32 kHz RC oscillator initialization
// Enable the internal 32 kHz RC oscillator
OSC.CTRL|=OSC_RC32KEN_bm;
// Wait for the internal 32 kHz RC oscillator to stabilize
while ((OSC.STATUS & OSC_RC32KRDY_bm)==0);

// Internal 32 MHz RC oscillator initialization
// Enable the internal 32 MHz RC oscillator
OSC.CTRL|=OSC_RC32MEN_bm;

// System clock prescaler A division factor: 2
// System clock prescalers B & C division factors: B:1, C:1
// ClkPer4: 16000,000 kHz
// ClkPer2: 16000,000 kHz
// ClkPer:  16000,000 kHz
// ClkCPU:  16000,000 kHz
n=(CLK.PSCTRL & (~(CLK_PSADIV_gm | CLK_PSBCDIV1_bm | CLK_PSBCDIV0_bm))) |
	CLK_PSADIV_2_gc | CLK_PSBCDIV_1_1_gc;
CCP=CCP_IOREG_gc;
CLK.PSCTRL=n;

// Internal 32 MHz RC osc. calibration reference clock source: 32.768 kHz Internal Osc.
OSC.DFLLCTRL&= ~(OSC_RC32MCREF_bm | OSC_RC2MCREF_bm);
// Enable the autocalibration of the internal 32 MHz RC oscillator
DFLLRC32M.CTRL|=DFLL_ENABLE_bm;

// Wait for the internal 32 MHz RC oscillator to stabilize
while ((OSC.STATUS & OSC_RC32MRDY_bm)==0);

// Select the system clock source: 32 MHz Internal RC Osc.
n=(CLK.CTRL & (~CLK_SCLKSEL_gm)) | CLK_SCLKSEL_RC32M_gc;
CCP=CCP_IOREG_gc;
CLK.CTRL=n;

// Disable the unused oscillators: 2 MHz, external clock/crystal oscillator, PLL
OSC.CTRL&= ~(OSC_RC2MEN_bm | OSC_XOSCEN_bm | OSC_PLLEN_bm);

// Peripheral Clock output: Disabled
PORTCFG.CLKEVOUT=(PORTCFG.CLKEVOUT & (~PORTCFG_CLKOUT_gm)) | PORTCFG_CLKOUT_OFF_gc;

// Restore interrupts enabled/disabled state
SREG=s;

}


